%% FM Quadraturdemodulator
% C. Roppel, FH Schmalkalden
% 11.08.2010
%
% Erluterungen siehe "Analoge Modulationsverfahren und Rundfunktechnik",
% Begleitmaterial zum Buch "Grundlagen der digitalen Kommunikationstechnik"
% Fachbuchverlag Leipzig, 2006
%
% Getestet mit Matlab 7.4.0 und 7.9.0
%

function xdemod = FM_IQdemod(fmsig, fc, fs, B_HF)
% fmsig: FM-Signal
% fc: Trgerrequenz in Hz
% fs: Abtastrate in Hz
% B_HF: Bandbreite des FM-Signals in Hz

% Quadraturdemodulation
t = (0:length(fmsig)-1)/fs;
losin = sin(2*pi*fc*t);
locos = cos(2*pi*fc*t);
xi = fmsig.*locos;
xq = -fmsig.*losin;
% TP-Filter, entfernt spektrale Anteile bei (2*fc)
iq_lp = fir1(256, (B_HF/2)/(fs/2));
xi = filter(iq_lp, 1, xi);
xq = filter(iq_lp, 1, xq);
% Entferne die ersten Samples (Einschwingvorgang des FIR-Filters)
xi = xi(500:end);
xq = xq(500:end);
ln = min(length(xi), length(xq));
xi = xi(1:ln);
xq = xq(1:ln);
% Normierte Amplitude
amp = xi.^2 + xq.^2;
amp = amp + mean(amp)/100;  % vermeide Division durch null
xi = xi./sqrt(amp);
xq = xq./sqrt(amp);
% FIR-Filter als Differenzierer
d = fdesign.differentiator(33);
diff = design(d,'firls');
xi_diff = filter(diff.Numerator, 1, xi);
xq_diff = filter(diff.Numerator, 1, xq);
% Kompensation der Verzgerung durch die FIR-Filter
xi_d = filter([zeros(1, 16) 1], 1, xi);
xq_d = filter([zeros(1, 16) 1], 1, xq);
xdemod = (xi_d .* xq_diff) - (xq_d .* xi_diff);

end
